@@ -1,6 +1,5 @@
 from pathlib import Path
 from subprocess import check_call, check_output
-from tempfile import TemporaryDirectory
 import logging
 import sys
 
@@ -22,10 +21,14 @@ cmeel_run()
 """
 
 
+class NonRelocatableError(Exception):
+    pass
+
+
 def build_wheel(wheel_directory, config_settings=None, metadata_directory=None):
     logging.info("CMake Wheel")
 
-    TEMP = Path(TemporaryDirectory(prefix="cmeel-").name)
+    TEMP = cmeel_config.temp_dir
     BUILD = TEMP / "bld"
     PREFIX = TEMP / "pfx"
     INSTALL = PREFIX / CMEEL_PREFIX
@@ -36,7 +39,12 @@ def build_wheel(wheel_directory, config_settings=None, metadata_directory=None):
         pyproject = tomli.load(f)
         CONF = pyproject["project"]
         SOURCE = pyproject["build-system"].get("source", ".")
-        RUN_TESTS = pyproject["build-system"].get("run_tests", True)
+        RUN_TESTS = pyproject["build-system"].get("run-tests", True)
+        RUN_TESTS_AFTER_INSTALL = pyproject["build-system"].get(
+            "run-tests-after-install", False
+        )
+        BUILD_NUMBER = pyproject["build-system"].get("build-number", 0)
+        CONFIGURE_ARGS = pyproject["build-system"].get("configure-args", [])
     DISTRIBUTION = CONF["name"].replace("-", "_")
 
     logging.info("build wheel")
@@ -50,21 +58,27 @@ def build_wheel(wheel_directory, config_settings=None, metadata_directory=None):
     # Configure
 
     logging.info("configure")
-    configure_args = cmeel_config.get_configure_args(CONF, INSTALL)
-    configure_env = cmeel_config.get_configure_env(CONF)
+    configure_args = cmeel_config.get_configure_args(CONF, INSTALL, CONFIGURE_ARGS)
+    configure_env = cmeel_config.get_configure_env()
     check_call(["cmake", "-S", SOURCE, "-B", BUILD] + configure_args, env=configure_env)
 
     logging.info("build")
     check_call(["cmake", "--build", BUILD, f"-j{cmeel_config.jobs}"])
 
-    if RUN_TESTS:
+    def run_tests():
         logging.info("test")
-        test_env = cmeel_config.get_test_env(CONF)
+        test_env = cmeel_config.get_test_env()
         check_call(["cmake", "--build", BUILD, "-t", "test"], env=test_env)
 
+    if RUN_TESTS and not RUN_TESTS_AFTER_INSTALL:
+        run_tests()
+
     logging.info("install")
     check_call(["cmake", "--build", BUILD, "-t", "install"])
 
+    if RUN_TESTS and RUN_TESTS_AFTER_INSTALL:
+        run_tests()
+
     logging.info("fix relocatablization")
     for f in INSTALL.rglob("*.cmake"):
         check_call(["sed", "-i", f"s|{INSTALL}|${{PACKAGE_PREFIX_DIR}}|g", str(f)])
@@ -90,7 +104,7 @@ def build_wheel(wheel_directory, config_settings=None, metadata_directory=None):
             "\n".join(
                 [
                     "Metadata-Version: 2.1",
-                    f"Name: {DISTRIBUTION}",
+                    f"Name: {CONF['name']}",
                     f"Version: {CONF['version']}",
                     f"Summary: {CONF['description']}",
                     f"Home-page: {CONF['urls']['homepage']}",
@@ -134,9 +148,47 @@ def build_wheel(wheel_directory, config_settings=None, metadata_directory=None):
                 fe.write(EXECUTABLE)
             executable.chmod(0o755)
 
+    logging.info("check generated cmake files")
+    WRONG_DIRS = [
+        "/tmp/pip-build-env",
+        "/tmp/pip-req-build",
+        "/opt/_internal",
+        str(TEMP),
+    ]
+    for fc in INSTALL.glob("**/*.cmake"):
+        with fc.open() as f:
+            cmake_file = f.read()
+            if any(wrong_dir in cmake_file for wrong_dir in WRONG_DIRS):
+                lines = cmake_file.split("\n")
+                # Get indexes of of problematic lines
+                indexes = [
+                    idx
+                    for idx, line in enumerate(lines)
+                    if any(wrong_dir in line for wrong_dir in WRONG_DIRS)
+                ]
+                # Get lines at those indexes and around them to display
+                display = [
+                    f"{i}: {l}"
+                    for i, l in enumerate(lines)
+                    if any(idx in indexes for idx in (i - 2, i - 1, i, i + 1, i + 2))
+                ]
+                raise NonRelocatableError(
+                    f"{fc} references temporary paths:\n" + "\n".join(display)
+                )
+
     logging.info("wheel pack")
     name = check_output(
-        [sys.executable, "-m", "wheel", "pack", "-d", wheel_directory, PREFIX]
+        [
+            sys.executable,
+            "-m",
+            "wheel",
+            "pack",
+            "--build-number",
+            str(BUILD_NUMBER),
+            "-d",
+            wheel_directory,
+            PREFIX,
+        ]
     ).decode()
     name = name.split("/")[-1][:-6]
 
